@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2011  Lars Wirzenius
+# Copyright (C) 2009-2014  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 import cliapp
 
 
-__version__ = '1.6.1'
+__version__ = '1.8'
 
 
 
@@ -33,14 +33,14 @@ try:
 except ImportError:
     _obnam = DummyExtension()
 
-from pluginmgr import PluginManager
 
+# Exceptions defined by Obnam itself. They should all be a subclass
+# of obnamlib.ObnamError.
 
-class Error(cliapp.AppException):
-    pass
+from structurederror import StructuredError
 
+class ObnamError(StructuredError):
 
-class EncryptionError(Error):
     pass
 
 
@@ -82,14 +82,30 @@ from encryption import (generate_symmetric_key,
                         SecretKeyring,
                         encrypt_with_keyring,
                         decrypt_with_secret_keys,
-                        SymmetricKeyCache)
+                        SymmetricKeyCache,
+                        EncryptionError)
 
-from hooks import Hook, MissingFilterError, FilterHook, HookManager
+from hooks import (
+    Hook, MissingFilterError, NoFilterTagError, FilterHook, HookManager)
 from pluginbase import ObnamPlugin
-from vfs import VirtualFileSystem, VfsFactory, VfsTests
+from vfs import (
+    VirtualFileSystem,
+    VfsFactory,
+    VfsTests,
+    LockFail,
+    NEW_DIR_MODE,
+    NEW_FILE_MODE)
 from vfs_local import LocalFS
-from metadata import (read_metadata, set_metadata, Metadata, metadata_fields,
-                      metadata_verify_fields, encode_metadata, decode_metadata)
+from fsck_work_item import WorkItem
+from lockmgr import LockManager
+from forget_policy import ForgetPolicy
+from app import App, ObnamIOError, ObnamSystemError
+from humanise import humanise_duration, humanise_size, humanise_speed
+
+from repo_factory import (
+    RepositoryFactory,
+    UnknownRepositoryFormat,
+    UnknownRepositoryFormatWanted)
 from repo_interface import (
     RepositoryInterface,
     RepositoryInterfaceTests,
@@ -110,21 +126,55 @@ from repo_interface import (
     RepositoryChunkContentNotInIndexes,
     RepositoryChunkIndexesNotLocked,
     RepositoryChunkIndexesLockingFailed,
+    repo_key_name,
     REPO_CLIENT_TEST_KEY,
     REPO_GENERATION_TEST_KEY,
+    REPO_GENERATION_STARTED,
+    REPO_GENERATION_ENDED,
+    REPO_GENERATION_IS_CHECKPOINT,
+    REPO_GENERATION_FILE_COUNT,
+    REPO_GENERATION_TOTAL_DATA,
     REPO_FILE_TEST_KEY,
-    REPO_FILE_MTIME,
+    REPO_FILE_MODE,
+    REPO_FILE_MTIME_SEC,
+    REPO_FILE_MTIME_NSEC,
+    REPO_FILE_ATIME_SEC,
+    REPO_FILE_ATIME_NSEC,
+    REPO_FILE_NLINK,
+    REPO_FILE_SIZE,
+    REPO_FILE_UID,
+    REPO_FILE_USERNAME,
+    REPO_FILE_GID,
+    REPO_FILE_GROUPNAME,
+    REPO_FILE_SYMLINK_TARGET,
+    REPO_FILE_XATTR_BLOB,
+    REPO_FILE_BLOCKS,
+    REPO_FILE_DEV,
+    REPO_FILE_INO,
+    REPO_FILE_MD5,
     REPO_FILE_INTEGER_KEYS)
+
+#
+# Repository format dummy specific modules.
+#
+
 from repo_dummy import RepositoryFormatDummy
-from repo_fmt_6 import RepositoryFormat6
-from repo_tree import RepositoryTree
-from chunklist import ChunkList
-from clientlist import ClientList
-from checksumtree import ChecksumTree
-from clientmetadatatree import ClientMetadataTree
-from lockmgr import LockManager
-from repo import Repository, LockFail, BadFormat
-from forget_policy import ForgetPolicy
-from app import App
+
+#
+# Repository format 6 specific modules.
+#
+
+from metadata import (
+    Metadata,
+    read_metadata,
+    set_metadata,
+    SetMetadataError,
+    metadata_fields)
+from fmt_6.repo_fmt_6 import RepositoryFormat6
+from fmt_6.repo_tree import RepositoryTree
+from fmt_6.chunklist import ChunkList
+from fmt_6.clientlist import ClientList
+from fmt_6.checksumtree import ChecksumTree
+from fmt_6.clientmetadatatree import ClientMetadataTree
 
 __all__ = locals()
