@@ -1,4 +1,4 @@
-# Copyright 2012  Lars Wirzenius
+# Copyright 2012-2014  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,7 +65,9 @@ class LockManager(object):
                 self._fs.lock(lock_name, self.data)
             except obnamlib.LockFail:
                 if self._time() - started >= self.timeout:
-                    raise obnamlib.LockFail('Lock timeout: %s' % lock_name)
+                    raise obnamlib.LockFail(
+                        lock_name=lock_name,
+                        reason='timeout')
             else:
                 return
             self._sleep()
@@ -73,6 +75,15 @@ class LockManager(object):
     def _unlock_one(self, dirname):
         self._fs.unlock(self._lockname(dirname))
 
+    def is_locked(self, dirname):
+        '''Is the given directory locked?
+
+        Note the usual race conditions between testing and locking.
+
+        '''
+
+        return self._fs.exists(self._lockname(dirname))
+
     def lock(self, dirnames):
         '''Lock ALL the directories.'''
         we_locked = []
