@@ -1,4 +1,4 @@
-# Copyright (C) 2009  Lars Wirzenius
+# Copyright (C) 2009-2014  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,13 +14,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+import cliapp
+
 import obnamlib
 
 
-class ObnamPlugin(obnamlib.pluginmgr.Plugin):
+class ObnamPlugin(cliapp.Plugin):
 
     '''Base class for plugins in Obnam.'''
 
     def __init__(self, app):
         self.app = app
 
+    def disable(self):
+        pass
