@@ -1,4 +1,4 @@
-# Copyright (C) 2009  Lars Wirzenius
+# Copyright (C) 2009-2014  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,3 +33,5 @@ class ObnamPluginTests(unittest.TestCase):
     def test_has_an_app(self):
         self.assertEqual(self.plugin.app, self.fakeapp)
 
+    def test_disable_is_implemented(self):
+        self.plugin.disable()
