@@ -1,4 +1,4 @@
-# Copyright (C) 2011  Lars Wirzenius
+# Copyright (C) 2011-2014  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
                                   group=encryption_group)
         self.app.settings.string(['symmetric-key-bits'],
                                    'size of symmetric key, in bits',
+                                 metavar='BITS',
                                  group=encryption_group)
 
         self.tag = "encrypt1"
@@ -96,10 +97,10 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
         return int(self.app.settings['symmetric-key-bits'] or '256')
 
     def _write_file(self, repo, pathname, contents):
-        repo.fs.fs.write_file(pathname, contents)
+        repo.get_fs().write_file(pathname, contents)
 
     def _overwrite_file(self, repo, pathname, contents):
-        repo.fs.fs.overwrite_file(pathname, contents)
+        repo.get_fs().overwrite_file(pathname, contents)
 
     def toplevel_init(self, repo, toplevel):
         '''Initialize a new toplevel for encryption.'''
@@ -121,8 +122,6 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
         self._write_file(repo, os.path.join(toplevel, 'userkeys'), encrypted)
 
     def filter_read(self, encrypted, repo, toplevel):
-        if not self.keyid:
-            return encrypted
         symmetric_key = self.get_symmetric_key(repo, toplevel)
         return obnamlib.decrypt_symmetric(encrypted, symmetric_key)
 
@@ -135,13 +134,13 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
     def get_symmetric_key(self, repo, toplevel):
         key = self._symkeys.get(repo, toplevel)
         if key is None:
-            encoded = repo.fs.fs.cat(os.path.join(toplevel, 'key'))
+            encoded = repo.get_fs().cat(os.path.join(toplevel, 'key'))
             key = obnamlib.decrypt_with_secret_keys(encoded)
             self._symkeys.put(repo, toplevel, key)
         return key
 
     def read_keyring(self, repo, toplevel):
-        encrypted = repo.fs.fs.cat(os.path.join(toplevel, 'userkeys'))
+        encrypted = repo.get_fs().cat(os.path.join(toplevel, 'userkeys'))
         encoded = self.filter_read(encrypted, repo, toplevel)
         return obnamlib.Keyring(encoded=encoded)
 
@@ -186,10 +185,10 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
         '''List clients and their keys in the repository.'''
         if self.quit_if_unencrypted():
             return
-        repo = self.app.open_repository()
-        clients = repo.list_clients()
+        repo = self.app.get_repository_object()
+        clients = repo.get_client_names()
         for client in clients:
-            keyid = repo.clientlist.get_client_keyid(client)
+            keyid = repo.get_client_encryption_key_id(client)
             if keyid is None:
                 key_info = 'no key'
             else:
@@ -197,12 +196,12 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
             print client, key_info
 
     def _find_keys_and_toplevels(self, repo):
-        toplevels = repo.fs.listdir('.')
+        toplevels = repo.get_fs().listdir('.')
         keys = dict()
         tops = dict()
         for toplevel in [d for d in toplevels if d != 'metadata']:
             # skip files (e.g. 'lock') or empty directories
-            if not repo.fs.exists(os.path.join(toplevel, 'key')):
+            if not repo.get_fs().exists(os.path.join(toplevel, 'key')):
                 continue
             try:
                 userkeys = self.read_keyring(repo, toplevel)
@@ -227,7 +226,7 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
         '''List keys and the repository toplevels they're used in.'''
         if self.quit_if_unencrypted():
             return
-        repo = self.app.open_repository()
+        repo = self.app.get_repository_object()
         keys, tops = self._find_keys_and_toplevels(repo)
         for keyid in keys:
             print 'key: %s' % self._get_key_string(keyid)
@@ -238,7 +237,7 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
         '''List repository toplevel directories and their keys.'''
         if self.quit_if_unencrypted():
             return
-        repo = self.app.open_repository()
+        repo = self.app.get_repository_object()
         keys, tops = self._find_keys_and_toplevels(repo)
         for toplevel in tops:
             print 'toplevel: %s' % toplevel
@@ -248,21 +247,15 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
     _shared = ['chunklist', 'chunks', 'chunksums', 'clientlist']
 
     def _find_clientdirs(self, repo, client_names):
-        result = []
-        for client_name in client_names:
-            client_id = repo.clientlist.get_client_id(client_name)
-            if client_id:
-                result.append(repo.client_dir(client_id))
-            else:
-                logging.warning("client not found: %s" % client_name)
-        return result
+        return [repo.get_client_extra_data_directory(client_name)
+                for client_name in client_names]
 
     def add_key(self, args):
         '''Add a key to the repository.'''
         if self.quit_if_unencrypted():
             return
         self.app.settings.require('keyid')
-        repo = self.app.open_repository()
+        repo = self.app.get_repository_object()
         keyid = self.app.settings['keyid']
         key = obnamlib.get_public_key(keyid)
         clients = self._find_clientdirs(repo, args)
@@ -275,7 +268,7 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
         if self.quit_if_unencrypted():
             return
         self.app.settings.require('keyid')
-        repo = self.app.open_repository()
+        repo = self.app.get_repository_object()
         keyid = self.app.settings['keyid']
         clients = self._find_clientdirs(repo, args)
         for toplevel in self._shared + clients:
@@ -286,10 +279,10 @@ class EncryptionPlugin(obnamlib.ObnamPlugin):
         '''Remove client and its key from repository.'''
         if self.quit_if_unencrypted():
             return
-        repo = self.app.open_repository()
-        repo.lock_root()
+        repo = self.app.get_repository_object()
+        repo.lock_client_list()
         for client_name in args:
             logging.info('removing client %s' % client_name)
             repo.remove_client(client_name)
-        repo.commit_root()
+        repo.commit_client_list()
 
