@@ -1,4 +1,4 @@
-# Copyright (C) 2009  Lars Wirzenius <liw@liw.fi>
+# Copyright (C) 2009-2014  Lars Wirzenius <liw@liw.fi>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ import time
 import traceback
 import urlparse
 
+
 # As of 2010-07-10, Debian's paramiko package triggers
 # RandomPool_DeprecationWarning. This will eventually be fixed. Until
 # then, there is no point in spewing the warning to the user, who can't
@@ -41,6 +42,41 @@ with warnings.catch_warnings():
 import obnamlib
 
 
+class NoHostKeyError(obnamlib.ObnamError):
+
+    msg = 'No known host key for {hostname}'
+
+
+class NoHostKeyOfWantedTypeError(obnamlib.ObnamError):
+
+    msg = 'No known type {key_type} host key for {hostname}'
+
+
+class WrongHostKeyError(obnamlib.ObnamError):
+
+    msg = 'SSH server {hostname} offered wrong public key'
+
+
+class KeyAuthenticationError(obnamlib.ObnamError):
+
+    msg = "Can't authenticate to SSH server using key"
+
+
+class WrongURLSchemeError(obnamlib.ObnamError):
+
+    msg = 'SftpFS used with non-sftp URL: {url}'
+
+
+class InvalidPortError(obnamlib.ObnamError):
+
+    msg = 'Invalid port number {port} in {url}: {error}'
+
+
+class HardlinkError(obnamlib.ObnamError):
+
+    msg = 'Cannot hardlink on SFTP; sorry'
+
+
 def ioerror_to_oserror(method):
     '''Decorator to convert an IOError exception to OSError.
 
@@ -74,8 +110,10 @@ class SSHChannelAdapter(object):
         try:
             return os.read(self.proc.stdout.fileno(), count)
         except socket.error, e:
-            if e.args[0] in (errno.EPIPE, errno.ECONNRESET, errno.ECONNABORTED,
-                             errno.EBADF):
+            errnos = (
+                errno.EPIPE, errno.ECONNRESET, errno.ECONNABORTED,
+                errno.EBADF)
+            if e.args[0] in errnos:
                 # Connection has closed.  Paramiko expects an empty string in
                 # this case, not an exception.
                 return ''
@@ -226,7 +264,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
         known_keys = known_hosts.lookup(hostname)
         if known_keys is None:
             if self.settings['ssh-host-keys-check'] == 'yes':
-                raise obnamlib.Error('No known host key for %s' % hostname)
+                raise NoHostKeyError(hostname=hostname)
             logging.warning('No known host keys for %s; accepting offered key'
                             % hostname)
             return
@@ -234,15 +272,14 @@ class SftpFS(obnamlib.VirtualFileSystem):
         offered_type = offered_key.get_name()
         if not known_keys.has_key(offered_type):
             if self.settings['ssh-host-keys-check'] == 'yes':
-                raise obnamlib.Error('No known type %s host key for %s' %
-                                     (offered_type, hostname))
+                raise NoHostKeyOfWantedTypeError(
+                    key_type=offered_type, hostname=hostname)
             logging.warning('No known host key of type %s for %s; accepting '
                             'offered key' % (offered_type, hostname))
 
         known_key = known_keys[offered_type]
         if offered_key != known_key:
-            raise obnamlib.Error('SSH server %s offered wrong public key' %
-                                 hostname)
+            raise WrongHostKeyError(hostname=hostname)
 
         logging.debug('Host key for %s OK' % hostname)
 
@@ -255,7 +292,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
                 return
             except paramiko.SSHException:
                 pass
-        raise obnamlib.Error('Can\'t authenticate to SSH server using key.')
+        raise KeyAuthenticationError()
 
     def _find_auth_keys(self):
         if self.settings and self.settings['ssh-key']:
@@ -291,7 +328,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
         scheme, netloc, path, query, fragment = urlparse.urlsplit(baseurl)
 
         if scheme != 'sftp':
-            raise obnamlib.Error('SftpFS used with non-sftp URL: %s' % baseurl)
+            raise WrongURLSchemeError(url=baseurl)
 
         if '@' in netloc:
             user, netloc = netloc.split('@', 1)
@@ -306,10 +343,10 @@ class SftpFS(obnamlib.VirtualFileSystem):
                 try:
                     port = int(port)
                 except ValueError, e:
-                    msg = ('Invalid port number %s in %s: %s' %
-                            (port, baseurl, str(e)))
-                    logging.error(msg)
-                    raise obnamlib.Error(msg)
+                    exc = InvalidPortError(
+                        port=port, url=baseurl, error=str(e))
+                    logging.error(str(exc))
+                    raise exc
         else:
             host = netloc
             port = None
@@ -405,7 +442,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
         try:
             self.write_file(lockname, data)
         except OSError, e:
-            raise obnamlib.LockFail('Failure get lock %s' % lockname)
+            raise obnamlib.LockFail(lock_name=lockname, reason=str(e))
 
     def unlock(self, lockname):
         self._remove_if_exists(lockname)
@@ -434,16 +471,16 @@ class SftpFS(obnamlib.VirtualFileSystem):
         raise NotImplementedError('mknod on SFTP: %s' % pathname)
 
     @ioerror_to_oserror
-    def mkdir(self, pathname):
+    def mkdir(self, pathname, mode=obnamlib.NEW_DIR_MODE):
         self._delay()
-        self.sftp.mkdir(pathname)
+        self.sftp.mkdir(pathname, mode)
 
     @ioerror_to_oserror
     def makedirs(self, pathname):
         parent = os.path.dirname(pathname)
         if parent and parent != pathname and not self.exists(parent):
             self.makedirs(parent)
-        self.mkdir(pathname)
+        self.mkdir(pathname, obnamlib.NEW_DIR_MODE)
 
     @ioerror_to_oserror
     def rmdir(self, pathname):
@@ -511,7 +548,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
         self.sftp.utime(pathname, (atime_sec, mtime_sec))
 
     def link(self, existing_path, new_path):
-        raise obnamlib.Error('Cannot hardlink on SFTP. Sorry.')
+        raise HardlinkError()
 
     def readlink(self, symlink):
         self._delay()
@@ -528,7 +565,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
 
     def cat(self, pathname):
         self._delay()
-        f = self.open(pathname, 'r')
+        f = self.open(pathname, 'rb')
         f.prefetch()
         chunks = []
         while True:
@@ -543,7 +580,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
     @ioerror_to_oserror
     def write_file(self, pathname, contents):
         try:
-            f = self.open(pathname, 'wx')
+            f = self.open(pathname, 'wbx')
         except (IOError, OSError), e:
             # When the path to the file to be written does not
             # exist, we try to create the directories below. Note that
@@ -561,7 +598,7 @@ class SftpFS(obnamlib.VirtualFileSystem):
         f.close()
 
     def _tempfile(self, dirname):
-        '''Create a new file with a random name, return file handle and name.'''
+        '''Create a new file with a random name, return handle and name.'''
 
         if dirname:
             try:
@@ -578,7 +615,11 @@ class SftpFS(obnamlib.VirtualFileSystem):
             basename = 'tmp.%x' % i
             pathname = os.path.join(dirname, basename)
             try:
-                f = self.open(pathname, 'wx', bufsize=self.chunk_size)
+                f = self.open(pathname, 'wbx', bufsize=self.chunk_size)
+                # paramiko.SFTPClient doesn't allow setting the mode
+                # on creation, so we set it separately. This leaves a
+                # short window where the file is possible to open.
+                self.chmod_not_symlink(pathname, obnamlib.NEW_FILE_MODE)
             except OSError:
                 pass
             else:
@@ -606,54 +647,55 @@ class SftpPlugin(obnamlib.ObnamPlugin):
         ssh_group = obnamlib.option_group['ssh'] = 'SSH/SFTP'
         devel_group = obnamlib.option_group['devel']
 
-        self.app.settings.integer(['sftp-delay'],
-                                  'add an artificial delay (in milliseconds) '
-                                    'to all SFTP transfers',
-                                  group=devel_group)
+        self.app.settings.integer(
+            ['sftp-delay'],
+            'add an artificial delay (in milliseconds) to all SFTP transfers',
+            group=devel_group)
 
-        self.app.settings.string(['ssh-key'],
-                                 'use FILENAME as the ssh RSA private key for '
-                                    'sftp access (default is using keys known '
-                                    'to ssh-agent)',
-                                 metavar='FILENAME',
-                                 group=ssh_group)
+        self.app.settings.string(
+            ['ssh-key'],
+            'use FILENAME as the ssh RSA private key for sftp access '
+            '(default is using keys known to ssh-agent)',
+            metavar='FILENAME',
+            group=ssh_group)
 
-        self.app.settings.boolean(['strict-ssh-host-keys'],
-                                  'DEPRECATED, use --ssh-host-keys-check '
-                                    'instead',
-                                 group=ssh_group)
+        self.app.settings.boolean(
+            ['strict-ssh-host-keys'],
+            'DEPRECATED, use --ssh-host-keys-check instead',
+            group=ssh_group)
 
-        self.app.settings.choice(['ssh-host-keys-check'],
-                                 ['ssh-config', 'yes', 'no', 'ask'],
-                                  'If "yes", require that the ssh host key must '
-                                    'be known and correct to be accepted. If '
-                                    '"no", do not require that. If "ask", the '
-                                    'user is interactively asked to accept new '
-                                    'hosts. The default ("ssh-config") is to '
-                                    'rely on the settings of the underlying '
-                                    'SSH client',
-                                 metavar='VALUE',
-                                 group=ssh_group)
+        self.app.settings.choice(
+            ['ssh-host-keys-check'],
+            ['ssh-config', 'yes', 'no', 'ask'],
+            'If "yes", require that the ssh host key must '
+            'be known and correct to be accepted. If '
+            '"no", do not require that. If "ask", the '
+            'user is interactively asked to accept new '
+            'hosts. The default ("ssh-config") is to '
+            'rely on the settings of the underlying '
+            'SSH client',
+            metavar='VALUE',
+            group=ssh_group)
 
-        self.app.settings.string(['ssh-known-hosts'],
-                                 'filename of the user\'s known hosts file '
-                                    '(default: %default)',
-                                 metavar='FILENAME',
-                                 default=
-                                    os.path.expanduser('~/.ssh/known_hosts'),
-                                 group=ssh_group)
+        self.app.settings.string(
+            ['ssh-known-hosts'],
+            'filename of the user\'s known hosts file',
+            metavar='FILENAME',
+            default=os.path.expanduser('~/.ssh/known_hosts'),
+            group=ssh_group)
 
-        self.app.settings.string(['ssh-command'],
-                                 'alternative executable to be used instead '
-                                    'of "ssh" (full path is allowed, no '
-                                    'arguments may be added)',
-                                 metavar='EXECUTABLE',
-                                 group=ssh_group)
+        self.app.settings.string(
+            ['ssh-command'],
+            'alternative executable to be used instead '
+            'of "ssh" (full path is allowed, no '
+            'arguments may be added)',
+            metavar='EXECUTABLE',
+            group=ssh_group)
 
-        self.app.settings.boolean(['pure-paramiko'],
-                                 'do not use openssh even if available, '
-                                    'use paramiko only instead',
-                                  group=ssh_group)
+        self.app.settings.boolean(
+            ['pure-paramiko'],
+            'do not use openssh even if available, '
+            'use paramiko only instead',
+            group=ssh_group)
 
         self.app.fsf.register('sftp', SftpFS, settings=self.app.settings)
-
