@@ -1,4 +1,4 @@
-# Copyright 2013  Lars Wirzenius
+# Copyright 2013-2014  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -96,22 +96,31 @@ class DummyClient(object):
 
     def __init__(self, name):
         self.name = name
+        self.key_id = None
         self.generation_counter = Counter()
         self.data = LockableKeyValueStore()
 
+    def is_locked(self):
+        return self.data.locked
+
     def lock(self):
         if self.data.locked:
-            raise obnamlib.RepositoryClientLockingFailed(self.name)
+            raise obnamlib.RepositoryClientLockingFailed(
+                client_name=self.name)
         self.data.lock()
 
     def _require_lock(self):
         if not self.data.locked:
-            raise obnamlib.RepositoryClientNotLocked(self.name)
+            raise obnamlib.RepositoryClientNotLocked(client_name=self.name)
 
     def unlock(self):
         self._require_lock()
         self.data.unlock()
 
+    def force_lock(self):
+        if self.data.locked:
+            self.data.unlock()
+
     def commit(self):
         self._require_lock()
         self.data.set_value('current-generation', None)
@@ -131,7 +140,8 @@ class DummyClient(object):
     def create_generation(self):
         self._require_lock()
         if self.data.get_value('current-generation', None) is not None:
-            raise obnamlib.RepositoryClientGenerationUnfinished(self.name)
+            raise obnamlib.RepositoryClientGenerationUnfinished(
+                client_name=self.name)
         generation_id = (self.name, self.generation_counter.next())
         ids = self.data.get_value('generation-ids', [])
         self.data.set_value('generation-ids', ids + [generation_id])
@@ -166,7 +176,8 @@ class DummyClient(object):
     def _require_generation(self, gen_id):
         ids = self.data.get_value('generation-ids', [])
         if gen_id not in ids:
-            raise obnamlib.RepositoryGenerationDoesNotExist(self.name)
+            raise obnamlib.RepositoryGenerationDoesNotExist(
+                client_name=self.name, gen_id=gen_id)
 
     def get_generation_key(self, gen_id, key):
         return self.data.get_value(gen_id + (key,), '')
@@ -191,7 +202,8 @@ class DummyClient(object):
     def interpret_generation_spec(self, genspec):
         ids = self.data.get_value('generation-ids', [])
         if not ids:
-            raise obnamlib.RepositoryClientHasNoGenerations(self.name)
+            raise obnamlib.RepositoryClientHasNoGenerations(
+                client_name=self.name)
         if genspec == 'latest':
             if ids:
                 return ids[-1]
@@ -199,7 +211,8 @@ class DummyClient(object):
             gen_number = int(genspec)
             if (self.name, gen_number) in ids:
                 return (self.name, gen_number)
-        raise obnamlib.RepositoryGenerationDoesNotExist(self.name)
+        raise obnamlib.RepositoryGenerationDoesNotExist(
+            client_name=self.name, gen_id=genspec)
 
     def make_generation_spec(self, generation_id):
         name, gen_number = generation_id
@@ -240,16 +253,14 @@ class DummyClient(object):
     def _require_file(self, gen_id, filename):
         if not self.file_exists(gen_id, filename):
             raise obnamlib.RepositoryFileDoesNotExistInGeneration(
-                self.name, self.make_generation_spec(gen_id), filename)
-
-    _integer_keys = (
-        obnamlib.REPO_FILE_MTIME,
-    )
+                client_name=self.name,
+                genspec=self.make_generation_spec(gen_id),
+                filename=filename)
 
     def get_file_key(self, gen_id, filename, key):
         self._require_generation(gen_id)
         self._require_file(gen_id, filename)
-        if key in self._integer_keys:
+        if key in obnamlib.REPO_FILE_INTEGER_KEYS:
             default = 0
         else:
             default = ''
@@ -305,6 +316,7 @@ class DummyClient(object):
             children.append(candidate)
         return children
 
+
 class DummyClientList(object):
 
     def __init__(self):
@@ -328,7 +340,6 @@ class DummyClientList(object):
     def force(self):
         if self.data.locked:
             self.unlock()
-        self.lock()
 
     def _require_lock(self):
         if not self.data.locked:
@@ -340,28 +351,33 @@ class DummyClientList(object):
     def __getitem__(self, client_name):
         client = self.data.get_value(client_name, None)
         if client is None:
-            raise obnamlib.RepositoryClientDoesNotExist(client_name)
+            raise obnamlib.RepositoryClientDoesNotExist(
+                client_name=client_name)
         return client
 
     def add(self, client_name):
         self._require_lock()
         if self.data.get_value(client_name, None) is not None:
-            raise obnamlib.RepositoryClientAlreadyExists(client_name)
+            raise obnamlib.RepositoryClientAlreadyExists(
+                client_name=client_name)
         self.data.set_value(client_name, DummyClient(client_name))
 
     def remove(self, client_name):
         self._require_lock()
         if self.data.get_value(client_name, None) is None:
-            raise obnamlib.RepositoryClientDoesNotExist(client_name)
+            raise obnamlib.RepositoryClientDoesNotExist(
+                client_name=client_name)
         self.data.set_value(client_name, None)
 
     def rename(self, old_client_name, new_client_name):
         self._require_lock()
         client = self.data.get_value(old_client_name, None)
         if client is None:
-            raise obnamlib.RepositoryClientDoesNotExist(old_client_name)
+            raise obnamlib.RepositoryClientDoesNotExist(
+                client_name=old_client_name)
         if self.data.get_value(new_client_name, None) is not None:
-            raise obnamlib.RepositoryClientAlreadyExists(new_client_name)
+            raise obnamlib.RepositoryClientAlreadyExists(
+                client_name=new_client_name)
         self.data.set_value(old_client_name, None)
         self.data.set_value(new_client_name, client)
 
@@ -369,6 +385,20 @@ class DummyClientList(object):
         client_name, generation_number = gen_id
         return self[client_name]
 
+    def get_client_encryption_key_id(self, client_name):
+        client = self.data.get_value(client_name, None)
+        if client is None:
+            raise obnamlib.RepositoryClientDoesNotExist(
+                client_name=client_name)
+        return client.key_id
+
+    def set_client_encryption_key_id(self, client_name, key_id):
+        client = self.data.get_value(client_name, None)
+        if client is None:
+            raise obnamlib.RepositoryClientDoesNotExist(
+                client_name=client_name)
+        client.key_id = key_id
+
 
 class ChunkStore(object):
 
@@ -383,7 +413,7 @@ class ChunkStore(object):
 
     def get_chunk_content(self, chunk_id):
         if chunk_id not in self.chunks:
-            raise obnamlib.RepositoryChunkDoesNotExist(str(chunk_id))
+            raise obnamlib.RepositoryChunkDoesNotExist(chunk_id=str(chunk_id))
         return self.chunks[chunk_id]
 
     def has_chunk(self, chunk_id):
@@ -391,9 +421,12 @@ class ChunkStore(object):
 
     def remove_chunk(self, chunk_id):
         if chunk_id not in self.chunks:
-            raise obnamlib.RepositoryChunkDoesNotExist(str(chunk_id))
+            raise obnamlib.RepositoryChunkDoesNotExist(chunk_id=str(chunk_id))
         del self.chunks[chunk_id]
 
+    def get_chunk_ids(self):
+        return self.chunks.keys()
+
 
 class ChunkIndexes(object):
 
@@ -420,17 +453,22 @@ class ChunkIndexes(object):
     def force(self):
         if self.data.locked:
             self.unlock()
-        self.lock()
 
-    def put_chunk(self, chunk_id, chunk_content, client_id):
+    def prepare(self, chunk_content):
+        return chunk_content
+
+    def put_chunk(self, chunk_id, token_is_chunk_content, client_id):
         self._require_lock()
-        self.data.set_value(chunk_id, chunk_content)
+        self.data.set_value(chunk_id, token_is_chunk_content)
 
     def find_chunk(self, chunk_content):
+        chunk_ids = []
         for chunk_id, stored_content in self.data.items():
             if stored_content == chunk_content:
-                return chunk_id
-        raise obnamlib.RepositoryChunkContentNotInIndexes()
+                chunk_ids.append(chunk_id)
+        if not chunk_ids:
+            raise obnamlib.RepositoryChunkContentNotInIndexes()
+        return chunk_ids
 
     def remove_chunk(self, chunk_id, client_id):
         self._require_lock()
@@ -451,13 +489,20 @@ class RepositoryFormatDummy(obnamlib.RepositoryInterface):
         self._client_list = DummyClientList()
         self._chunk_store = ChunkStore()
         self._chunk_indexes = ChunkIndexes()
+        self._fs = None
+
+    def get_fs(self):
+        return self._fs
 
     def set_fs(self, fs):
-        pass
+        self._fs = fs
 
     def init_repo(self):
         pass
 
+    def close(self):
+        pass
+
     def get_client_names(self):
         return self._client_list.names()
 
@@ -470,6 +515,9 @@ class RepositoryFormatDummy(obnamlib.RepositoryInterface):
     def commit_client_list(self):
         self._client_list.commit()
 
+    def got_client_list_lock(self):
+        return self._client_list.data.locked
+
     def force_client_list_lock(self):
         self._client_list.force()
 
@@ -482,6 +530,15 @@ class RepositoryFormatDummy(obnamlib.RepositoryInterface):
     def rename_client(self, old_client_name, new_client_name):
         self._client_list.rename(old_client_name, new_client_name)
 
+    def get_client_encryption_key_id(self, client_name):
+        return self._client_list.get_client_encryption_key_id(client_name)
+
+    def set_client_encryption_key_id(self, client_name, key_id):
+        self._client_list.set_client_encryption_key_id(client_name, key_id)
+
+    def client_is_locked(self, client_name):
+        return self._client_list[client_name].is_locked()
+
     def lock_client(self, client_name):
         self._client_list[client_name].lock()
 
@@ -491,6 +548,12 @@ class RepositoryFormatDummy(obnamlib.RepositoryInterface):
     def commit_client(self, client_name):
         self._client_list[client_name].commit()
 
+    def got_client_lock(self, client_name):
+        return self._client_list[client_name].data.locked
+
+    def force_client_lock(self, client_name):
+        self._client_list[client_name].force_lock()
+
     def get_allowed_client_keys(self):
         return [obnamlib.REPO_CLIENT_TEST_KEY]
 
@@ -500,17 +563,26 @@ class RepositoryFormatDummy(obnamlib.RepositoryInterface):
     def set_client_key(self, client_name, key, value):
         if key not in self.get_allowed_client_keys():
             raise obnamlib.RepositoryClientKeyNotAllowed(
-                self.format, client_name, key)
+                format=self.format,
+                client_name=client_name,
+                key_name=obnamlib.repo_key_name(key))
         self._client_list[client_name].set_key(key, value)
 
     def get_client_generation_ids(self, client_name):
         return self._client_list[client_name].get_generation_ids()
 
+    def get_client_extra_data_directory(self, client_name):
+        return client_name
+
     def create_generation(self, client_name):
         return self._client_list[client_name].create_generation()
 
     def get_allowed_generation_keys(self):
-        return [obnamlib.REPO_GENERATION_TEST_KEY]
+        return [
+            obnamlib.REPO_GENERATION_TEST_KEY,
+            obnamlib.REPO_GENERATION_STARTED,
+            obnamlib.REPO_GENERATION_ENDED,
+            ]
 
     def get_generation_key(self, generation_id, key):
         client = self._client_list.get_client_by_generation_id(generation_id)
@@ -520,7 +592,9 @@ class RepositoryFormatDummy(obnamlib.RepositoryInterface):
         client = self._client_list.get_client_by_generation_id(generation_id)
         if key not in self.get_allowed_generation_keys():
             raise obnamlib.RepositoryGenerationKeyNotAllowed(
-                self.format, client.name, key)
+                format=self.format,
+                client_name=client.name,
+                key_name=obnamlib.repo_key_name(key))
         return client.set_generation_key(generation_id, key, value)
 
     def remove_generation(self, generation_id):
@@ -555,18 +629,41 @@ class RepositoryFormatDummy(obnamlib.RepositoryInterface):
         client = self._client_list.get_client_by_generation_id(generation_id)
         if key not in self.get_allowed_file_keys():
             raise obnamlib.RepositoryFileKeyNotAllowed(
-                self.format, client.name, key)
+                format=self.format,
+                client_name=client.name,
+                key_name=obnamlib.repo_key_name(key))
         return client.get_file_key(generation_id, filename, key)
 
     def set_file_key(self, generation_id, filename, key, value):
         client = self._client_list.get_client_by_generation_id(generation_id)
         if key not in self.get_allowed_file_keys():
             raise obnamlib.RepositoryFileKeyNotAllowed(
-                self.format, client.name, key)
+                format=self.format,
+                client_name=client.name,
+                key_name=obnamlib.repo_key_name(key))
         client.set_file_key(generation_id, filename, key, value)
 
     def get_allowed_file_keys(self):
-        return [obnamlib.REPO_FILE_TEST_KEY, obnamlib.REPO_FILE_MTIME]
+        return [
+            obnamlib.REPO_FILE_TEST_KEY,
+            obnamlib.REPO_FILE_MODE,
+            obnamlib.REPO_FILE_MTIME_SEC,
+            obnamlib.REPO_FILE_MTIME_NSEC,
+            obnamlib.REPO_FILE_ATIME_SEC,
+            obnamlib.REPO_FILE_ATIME_NSEC,
+            obnamlib.REPO_FILE_NLINK,
+            obnamlib.REPO_FILE_SIZE,
+            obnamlib.REPO_FILE_UID,
+            obnamlib.REPO_FILE_GID,
+            obnamlib.REPO_FILE_BLOCKS,
+            obnamlib.REPO_FILE_DEV,
+            obnamlib.REPO_FILE_INO,
+            obnamlib.REPO_FILE_USERNAME,
+            obnamlib.REPO_FILE_GROUPNAME,
+            obnamlib.REPO_FILE_SYMLINK_TARGET,
+            obnamlib.REPO_FILE_XATTR_BLOB,
+            obnamlib.REPO_FILE_MD5,
+            ]
 
     def get_file_chunk_ids(self, generation_id, filename):
         client = self._client_list.get_client_by_generation_id(generation_id)
@@ -596,6 +693,9 @@ class RepositoryFormatDummy(obnamlib.RepositoryInterface):
     def remove_chunk(self, chunk_id):
         self._chunk_store.remove_chunk(chunk_id)
 
+    def get_chunk_ids(self):
+        return self._chunk_store.get_chunk_ids()
+
     def lock_chunk_indexes(self):
         self._chunk_indexes.lock()
 
@@ -605,18 +705,26 @@ class RepositoryFormatDummy(obnamlib.RepositoryInterface):
     def commit_chunk_indexes(self):
         self._chunk_indexes.commit()
 
+    def got_chunk_indexes_lock(self):
+        return self._chunk_indexes.data.locked
+
     def force_chunk_indexes_lock(self):
         self._chunk_indexes.force()
 
-    def put_chunk_into_indexes(self, chunk_id, chunk_content, client_id):
-        self._chunk_indexes.put_chunk(chunk_id, chunk_content, client_id)
+    def prepare_chunk_for_indexes(self, chunk_content):
+        return self._chunk_indexes.prepare(chunk_content)
 
-    def find_chunk_id_by_content(self, chunk_content):
+    def put_chunk_into_indexes(self, chunk_id, token, client_id):
+        self._chunk_indexes.put_chunk(chunk_id, token, client_id)
+
+    def find_chunk_ids_by_content(self, chunk_content):
         return self._chunk_indexes.find_chunk(chunk_content)
 
     def remove_chunk_from_indexes(self, chunk_id, client_id):
         return self._chunk_indexes.remove_chunk(chunk_id, client_id)
 
-    def get_fsck_work_item(self):
-        return 'this pretends to be a work item'
+    def validate_chunk_content(self, chunk_id):
+        return None
 
+    def get_fsck_work_items(self):
+        return []
