@@ -1,4 +1,4 @@
-# Copyright 2010  Lars Wirzenius
+# Copyright 2010-2014  Lars Wirzenius
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -85,11 +85,11 @@ class ByteSizeParserTests(unittest.TestCase):
 
     def test_size_syntax_error_includes_input_string(self):
         text = 'asdf asdf'
-        e = obnamlib.SizeSyntaxError(text)
+        e = obnamlib.SizeSyntaxError(size=text)
         self.assert_(text in str(e), str(e))
 
     def test_unit_name_error_includes_input_string(self):
         text = 'asdf asdf'
-        e = obnamlib.UnitNameError(text)
+        e = obnamlib.UnitNameError(unit=text)
         self.assert_(text in str(e), str(e))
 
