@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2010  Lars Wirzenius <liw@liw.fi>
+# Copyright (C) 2008-2014  Lars Wirzenius <liw@liw.fi>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,43 @@ import urlparse
 import obnamlib
 
 
+# Modes (permissions) for new directories and files. We use minimal
+# permissions to avoid allowing access to files being restored until
+# their original permissions have been set.
+#
+# Directories are RWX for owner, nothing for anyone else. We need
+# the R to be able to do listdir, W to create and remove files, and
+# X to access anything in the directory. We _could_ do without R,
+# but then everything that needs to later read needs to add the R
+# manually, and that's most things, so it's much easier to just add
+# R to everything. We can't do without W and X.
+#
+# Files are RW for owner, nothing for anyone else. Again, we could
+# do without R in some cases, but they're few enough that it's easier
+# to give R to everything. In any case, if an attacker has gained
+# access to our UID, they can already chmod to add R and then they
+# can read anyway.
+
+NEW_DIR_MODE = 0700
+NEW_FILE_MODE = 0600
+
+
+class URLSchemeAlreadyRegisteredError(obnamlib.ObnamError):
+
+    msg = 'VFS URL scheme {scheme} already registered'
+
+
+class UnknownVFSError(obnamlib.ObnamError):
+
+    msg = 'Unknown VFS type: {url}'
+
+
+class LockFail(obnamlib.ObnamError):
+
+    msg = "Couldn't create lock {lock_name}: {reason}"
+
+
+
 class VirtualFileSystem(object):
 
     '''A virtual filesystem interface.
@@ -291,7 +328,7 @@ class VfsFactory:
 
     def register(self, scheme, implementation, **kwargs):
         if scheme in self.implementations:
-            raise obnamlib.Error('URL scheme %s already registered' % scheme)
+            raise URLSchemeAlreadyRegisteredError(scheme=scheme)
         self.implementations[scheme] = (implementation, kwargs)
 
     def new(self, url, create=False):
@@ -300,7 +337,7 @@ class VfsFactory:
         if scheme in self.implementations:
             klass, kwargs = self.implementations[scheme]
             return klass(url, create=create, **kwargs)
-        raise obnamlib.Error('Unknown VFS type %s' % url)
+        raise UnknownVFSError(url=url)
 
 
 class VfsTests(object): # pragma: no cover
@@ -529,7 +566,8 @@ class VfsTests(object): # pragma: no cover
         st = self.fs.lstat('.')
         for field in obnamlib.metadata_fields:
             if field.startswith('st_'):
-                self.assert_(hasattr(st, field), 'stat must return %s' % field)
+                self.assertTrue(
+                    hasattr(st, field), 'stat must return %s' % field)
 
     def test_lstat_returns_right_filetype_for_directory(self):
         st = self.fs.lstat('.')
